# ===================================================================
#
# Copyright (c) 2021, Legrandin <helderijs@gmail.com>
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions
# are met:
#
# 1. Redistributions of source code must retain the above copyright
#    notice, this list of conditions and the following disclaimer.
# 2. Redistributions in binary form must reproduce the above copyright
#    notice, this list of conditions and the following disclaimer in
#    the documentation and/or other materials provided with the
#    distribution.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
# "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
# LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
# FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
# COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
# INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
# BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
# LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
# CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
# LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN
# ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
# POSSIBILITY OF SUCH DAMAGE.
# ===================================================================

import os
from setuptools import setup, find_packages

project_name = "pycryptodome-test-vectors"
project_dir = "pycryptodome_test_vectors"

with open(os.path.join(project_dir, "__init__.py")) as init_root:
    for line in init_root:
        if line.startswith("__version__"):
            version_string = eval(line.split("=")[1])

longdesc = """
This package contains an extensive set of test vectors
to verify the PyCryptodome cryptographic library.

PyCryptdome can be installed with either the ``pycryptodome``
package (``Crypto`` namespace) or
the ``pycryptodomex`` package (``Cryptodome`` namespace).

For more information, see the `homepage`_.

.. _`homepage`: http://www.pycryptodome.org
"""

setup(
    name=project_name,
    version=version_string,
    description='Test vectors for PyCryptodome',
    url='https://www.pycryptodome.org',
    author='Helder Eijs',
    author_email="helderijs@gmail.com",
    platforms='Posix; MacOS X; Windows',
    zip_safe=False,
    packages=find_packages(),
    include_package_data=True,
    license="BSD, Apache",
    long_description=longdesc,
    options={'bdist_wheel':{'universal':True}},
)



#!/usr/bin/python

import argparse

declaration = """\
/* This file was automatically generated, do not edit */
#include "common.h"
extern const unsigned {0}_n_tables;
extern const unsigned {0}_window_size;
extern const unsigned {0}_points_per_table;
extern const uint64_t {0}_tables[{1}][{2}][2][{3}];
"""

definition = """\
/* This file was automatically generated, do not edit */
#include "common.h"
const unsigned {0}_n_tables = {1};
const unsigned {0}_window_size = {2};
const unsigned {0}_points_per_table = {3};
/* {4} */
/* Table size: {5} kbytes */
const uint64_t {0}_tables[{1}][{3}][2][{6}] = {{\
"""

point = """\
  {{ /* Point #{0} */
    {{ {1} }},
    {{ {2} }}
  }}{3}\
"""

parser = argparse.ArgumentParser()
parser.add_argument("curve")
parser.add_argument("window_size", type=int)
parser.add_argument("basename")
args = parser.parse_args()

if args.curve == "p256":
    bits = 256
    p = 0xffffffff00000001000000000000000000000000ffffffffffffffffffffffff
    Gx = 0x6b17d1f2e12c4247f8bce6e563a440f277037d812deb33a0f4a13945d898c296
    Gy = 0x4fe342e2fe1a7f9b8ee7eb4a7c0f9e162bce33576b315ececbb6406837bf51f5
    msg = "Affine coordinates in Montgomery form"
elif args.curve == "p384":
    bits = 384
    p = 0xfffffffffffffffffffffffffffffffffffffffffffffffffffffffffffffffeffffffff0000000000000000ffffffff
    Gx = 0xaa87ca22be8b05378eb1c71ef320ad746e1d3b628ba79b9859f741e082542a385502f25dbf55296c3a545e3872760aB7
    Gy = 0x3617de4a96262c6f5d9e98bf9292dc29f8f41dbd289a147ce9da3113b5f0b8c00a60b1ce1d7e819d7a431d7c90ea0e5F
    msg = "Affine coordinates in Montgomery form"
elif args.curve == "p521":
    bits = 521
    p = 0x000001ffffffffffffffffffffffffffffffffffffffffffffffffffffffffffffffffffffffffffffffffffffffffffffffffffffffffffffffffffffffffffffffffff
    Gx = 0x000000c6858e06b70404e9cd9e3ecb662395b4429c648139053fb521f828af606b4d3dbaa14b5e77efe75928fe1dc127a2ffa8de3348b3c1856a429bf97e7e31c2e5bd66
    Gy = 0x0000011839296a789a3bc0045c8a5fb42c7d1bd998f54449579b446817afbd17273e662c97ee72995ef42640c550b9013fad0761353c7086a272c24088be94769fd16650
    msg = "Affine coordinates in plain form (not Montgomery)"
else:
    raise ValueError("Unsupported curve: " + args.curve)


c_file = open(args.basename + ".c", "wt")
h_file = open(args.basename + ".h", "wt")

words = (bits + 63) // 64
window_size = args.window_size
points_per_table = 2**window_size
n_tables = (bits + window_size - 1) // window_size
byte_size = n_tables * points_per_table * 2 * (bits // 64) * (64 // 8) // 1024
G = Gx, Gy


def double(X1, Y1):
    if X1 == 0 and Y1 == 0:
        return (0, 0)

    XX = pow(X1, 2, p)
    w = -3 + 3 * XX
    Y1Y1 = pow(Y1, 2, p)
    R = 2 * Y1Y1
    sss = 4 * Y1 * R
    RR = pow(R, 2, p)
    B = pow(X1 + R, 2, p) - XX - RR
    h = pow(w, 2, p) - 2 * B
    X3 = 2 * h * Y1 % p
    Y3 = w * (B - h) - 2 * RR % p
    Z3 = sss

    Z3inv = pow(Z3, p - 2, p)
    x3 = X3 * Z3inv % p
    y3 = Y3 * Z3inv % p
    return (x3, y3)


def add(X1, Y1, X2, Y2):
    if X1 == 0 and Y1 == 0:
        return (X2, Y2)
    if X1 == X2 and Y1 == Y2:
        return double(X1, Y1)
    if X1 == X2 and (Y1 + Y2) % p == 0:
        return (0, 0)

    u = Y2 - Y1
    uu = pow(u, 2, p)
    v = X2 - X1
    vv = pow(v, 2, p)
    vvv = v * vv % p
    R = vv * X1 % p
    A = uu - vvv - 2 * R
    X3 = v * A % p
    Y3 = (u * (R - A) - vvv * Y1) % p
    Z3 = vvv

    Z3inv = pow(Z3, p - 2, p)
    x3 = X3 * Z3inv % p
    y3 = Y3 * Z3inv % p
    return (x3, y3)


def get64(z, words):
    """Return a C string with the number encoded into 64-bit words"""

    # Convert to Montgomery form, but only if it's not P521
    if words != 9:
        R = 2**(words * 64)
        x = z * R % p
    else:
        x = z

    result = []
    for _ in range(words):
        masked = x & ((1 << 64) - 1)
        result.append("0x%016XULL" % masked)
        x >>= 64
    return ",".join(result)


# Create table with points 0, G, 2G, 3G, .. (2**window_size-1)G
window = [(0, 0)]
for _ in range(points_per_table - 1):
    new_point = add(*window[-1], *G)
    window.append(new_point)

print(declaration.format(args.curve, n_tables, points_per_table, words), file=h_file)
print(definition.format(args.curve, n_tables, window_size, points_per_table, msg,
                    byte_size, words), file=c_file)

for i in range(n_tables):
    print(" { /* Table #%u */" % i, file=c_file)
    for j, w in enumerate(window):
        endc = "" if (j == points_per_table - 1) else ","
        print(point.format(j, get64(w[0], words), get64(w[1], words), endc),
              file=c_file)
    endc = "" if (i == n_tables - 1) else ","
    print(" }%s" % endc, file=c_file)

    # Move from G to G*2^{w}
    for j in range(window_size):
        G = double(*G)

    # Update window
    for j in range(1, points_per_table):
        window[j] = add(*window[j-1], *G)

print("};", file=c_file)



#!/usr/bin/env python3

import time
from Crypto.PublicKey import RSA
from Crypto.Math import Numbers
from Crypto.Math.Numbers import Integer
from Crypto.Util.number import long_to_bytes
from Crypto.Util._raw_api import (load_pycryptodome_raw_lib,
                                  create_string_buffer,
                                  c_size_t)

ITER = 100

print(Numbers._implementation)

rsa_pem="""-----BEGIN RSA PRIVATE KEY-----
MIIEpAIBAAKCAQEA1maRsgBxvk1m1LcQMrN/oAfPq/V5/LkeUL/CdTs/DOe+dOIW
rvfibUrhgLwg170+qIpsv2+HOA5hPIl5tbBDsgCo/4hWo7EodeNumKdWnzhS0Cjp
Z1UQALAsGen6UugxFbiTCaq7Hhzx4stjadY31Gd1zkUj6jH2StJ5TLw2XdijXgB+
07V2lYd/vxAtvrizISSROY5JQxTpNyaSbhOD+Ku1iJvqlU64wMocYsjp2D9BiICV
xeZF7W0yUV/gxYwTaMrYRpThjaQ2aMb0PmHXybymM93Np671t5vDltSp9I4qmr4I
NsxFXkNTBTVyKOk9JartRrlS3vrg9XM5vyb1qQIDAQABAoIBACzgr2KJAUYKQZoI
75UNSYuf1vJxoaUqwpO4b+XGDv6Oi6k/oevh6z1hTS57Moy2CiWRRA4WNEGhkOzx
Ac7sJF9gD//c8/WzoXp7rqy5akJNsdfsmF6OyZi7R5/s//7Wp1+akPyXBi/ZczA7
zoVa17jYJyqUAl6FMr6aq9VKGD8wNTjSp+YhtBMdWegjpGJfOb19UY13hPfDqPGQ
Ydp0l0/0L6HAY97C25fUYeKRp9bnIXCKUineFmwSRjYzcoVOJ/PwiuJ0vBa/0gWw
KKTYE4ZJRDPVFt+7NfSVrLpeTh0YQ8s8MSm2ZCqF/HJEzlhF+sBxx/Yi5O4SrEP6
vuqgzQECgYEA/E9vn6XKnJXX3hiy0XVsUQK+Ana2xn63TxmiMJtrrUjD5WCiy9kD
eTU82igdg9JJGBWOd7mkSAWxVZT9MPaQ4/Vz6ggoZcPHZokmVP7qPePNmFXX6Jdz
hQx8HD0QcwwaYbIJ91WdoAP/cEuu+o+WPKB0CDYWrv3Dx9eTircemlECgYEA2Yk2
WXkOdD2KlnZgHNYp23F2XP6dF4Ny5n478thlJXdexRwRDSKCN/41jfC0FlPpZBcF
7EFTo+p7lhyJVxPBWSOIhJArMe98bdWoo+6TBuAPLisu7+i+HVRaoyHf0x3t7ViG
JqQOGjHN0G57cgSCrpaFwE2bAZ4dkA27M8Oy99kCgYEAw+HoB0nvwyGSNht2uKcx
MLOwULlZrUEzj3WXNaV0M1QKwkoEGb6hs7hhRf1e7LiVht01fj3iDQheZNMGvryu
QEyPcWJj+p3EcRaJa/N8aBAzzdDXjvwF84V91W6TFr6OvMo8colFlrWD2urnLh/L
w8XOT5Guiqz5Em2LXmZMnAECgYEAuZ1Ksq2Il8arKhd3iyNyM7xssozOnfGbaPDt
VhkutPlV8/ou0nZPhldyetqXzzVqP+0lMKHNLGA3c66Fwbcpk1Wudu5M7R7bnRxh
+P7olUU5rrtKIYsGLSB89hVBVnKDQbH3RaFWJyO36dFbo74Vg8ML/To6uPahYvlU
cqbZXoECgYAOybAyRS9vivcYoKywxtI0aU1HHibPrUl3izYLfDxfWgA0S6IGcBg1
Sx6XG8GYbzkouGb30yjIpn7JB1147DH1oxf8wcHXbAsgNX4IC6rQ1Iwef9P4pORF
icKO95pcPRhmfzuqfhEu/d/ZYjabao95baBHcrRxEbXZtjg88KVXKg==
-----END RSA PRIVATE KEY-----
"""

c_defs = """
int monty_pow(const uint8_t *base,
               const uint8_t *exp,
               const uint8_t *modulus,
               uint8_t       *out,
               size_t len,
               uint64_t seed);
"""

_raw_montgomery = load_pycryptodome_raw_lib("Crypto.Math._montgomery", c_defs)

key = RSA.import_key(rsa_pem)
message = 0xFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
SIZE = key.size_in_bytes()

# -----------------------------------------------------------------
start = time.time()
for x in range(ITER):
	result_cpython = pow(message, key.d, key.n)
end = time.time()
print("CPython =", end-start)

# -----------------------------------------------------------------
base_b = long_to_bytes(message, SIZE)
exp_b = long_to_bytes(key.d, SIZE)
modulus_b = long_to_bytes(key.n, SIZE)
out = create_string_buffer(SIZE)

start = time.time()
for _ in range(ITER):
    _raw_montgomery.monty_pow(
                base_b,
                exp_b,
                modulus_b,
                out,
                c_size_t(SIZE),
                32
                )
end = time.time()
my_time = end-start
print("Custom modexp =", my_time)

# -----------------------------------------------------------------
mg = Integer(message)
md = Integer(key.d)
mn = Integer(key.n)
start = time.time()
for x in range(ITER):
	result_gmp = pow(mg, md, mn)
end = time.time()
gmp_time = end - start
print("GMP =", gmp_time)

# -----------------------------------------------------------------
print("%.2f%%" % float((my_time/gmp_time-1)*100), "slower")



"""Make unit test for ladder_step() in x25519.c"""

from common import counter, make_main, split64, bin2int
from hashlib import sha256
import struct


def ref(x2, z2, x3, z3, x1):
    mod = 2**255 - 19

    x4 = (x2**2 - z2**2)**2
    z4 = 4*x2*z2*(x2**2 + 486662*x2*z2 + z2**2)
    x5 = 4*((x2*x3 - z2*z3)**2)
    z5 = 4*((x2*z3 - z2*x3)**2)*x1

    return x4 % mod, z4 % mod, x5 % mod, z5 % mod


def make_test_max():

    v = ["0x%08X" % (2**26-1)] * 10

    modulus = 2**255 - 19
    base = [0, 26, 51, 77, 102, 128, 153, 179, 204, 230]
    n = 0
    for i in range(10):
        n += (2**26 - 1) * (2**(base[i]))
    n %= modulus
    x2_out, z2_out, x3_out, z3_out = ref(n, n, n, n, n)

    # Output
    words = split64(x2_out)
    x2outx = words + ["0"] * (4 - len(words))
    words = split64(z2_out)
    z2outx = words + ["0"] * (4 - len(words))
    words = split64(x3_out)
    x3outx = words + ["0"] * (4 - len(words))
    words = split64(z3_out)
    z3outx = words + ["0"] * (4 - len(words))

    print("")
    print("void test_%d(void) {" % next(counter))
    print("    uint32_t x2[10] = { " + ",".join(v) + " };")
    print("    uint32_t z2[10] = { " + ",".join(v) + " };")
    print("    uint32_t x3[10] = { " + ",".join(v) + " };")
    print("    uint32_t z3[10] = { " + ",".join(v) + " };")
    print("    uint32_t xp[10] = { " + ",".join(v) + " };")
    print("    const uint64_t x2_out_ref[4] = {" + ", ".join(x2outx) + "};")
    print("    const uint64_t z2_out_ref[4] = {" + ", ".join(z2outx) + "};")
    print("    const uint64_t x3_out_ref[4] = {" + ", ".join(x3outx) + "};")
    print("    const uint64_t z3_out_ref[4] = {" + ", ".join(z3outx) + "};")
    print("    uint64_t x2_out[4] = { 0 };")
    print("    uint64_t z2_out[4] = { 0 };")
    print("    uint64_t x3_out[4] = { 0 };")
    print("    uint64_t z3_out[4] = { 0 };")

    print("")

    print("    ladder_step(x2, z2, x3, z3, xp);")
    print("    convert_le25p5_to_le64(x2_out, x2);")
    print("    convert_le25p5_to_le64(z2_out, z2);")
    print("    convert_le25p5_to_le64(x3_out, x3);")
    print("    convert_le25p5_to_le64(z3_out, z3);")
    print("    reduce_25519_le64(x2_out);")
    print("    reduce_25519_le64(z2_out);")
    print("    reduce_25519_le64(x3_out);")
    print("    reduce_25519_le64(z3_out);")

    print("")

    print("    assert(0 == memcmp(x2_out, x2_out_ref, sizeof x2_out));")
    print("    assert(0 == memcmp(z2_out, z2_out_ref, sizeof z2_out));")
    print("    assert(0 == memcmp(x3_out, x3_out_ref, sizeof x3_out));")
    print("    assert(0 == memcmp(z3_out, z3_out_ref, sizeof z3_out));")
    print("}")


def make_test(x2, z2, x3, z3, xp):

    x2_out, z2_out, x3_out, z3_out = ref(x2, z2, x3, z3, xp)

    # Input
    words = split64(x2)
    x2x = words + ["0"] * (4 - len(words))
    words = split64(z2)
    z2x = words + ["0"] * (4 - len(words))
    words = split64(x3)
    x3x = words + ["0"] * (4 - len(words))
    words = split64(z3)
    z3x = words + ["0"] * (4 - len(words))
    words = split64(xp)
    xpx = words + ["0"] * (4 - len(words))

    # Output
    words = split64(x2_out)
    x2outx = words + ["0"] * (4 - len(words))
    words = split64(z2_out)
    z2outx = words + ["0"] * (4 - len(words))
    words = split64(x3_out)
    x3outx = words + ["0"] * (4 - len(words))
    words = split64(z3_out)
    z3outx = words + ["0"] * (4 - len(words))

    print("")
    print("void test_%d(void) {" % next(counter))
    print("    const uint64_t x2_in[4] = {" + ", ".join(x2x) + "};")
    print("    const uint64_t z2_in[4] = {" + ", ".join(z2x) + "};")
    print("    const uint64_t x3_in[4] = {" + ", ".join(x3x) + "};")
    print("    const uint64_t z3_in[4] = {" + ", ".join(z3x) + "};")
    print("    const uint64_t xp_in[4] = {" + ", ".join(xpx) + "};")

    print("    const uint64_t x2_out_ref[4] = {" + ", ".join(x2outx) + "};")
    print("    const uint64_t z2_out_ref[4] = {" + ", ".join(z2outx) + "};")
    print("    const uint64_t x3_out_ref[4] = {" + ", ".join(x3outx) + "};")
    print("    const uint64_t z3_out_ref[4] = {" + ", ".join(z3outx) + "};")

    print("    uint32_t x2[10] = { 0 };")
    print("    uint32_t z2[10] = { 0 };")
    print("    uint32_t x3[10] = { 0 };")
    print("    uint32_t z3[10] = { 0 };")
    print("    uint32_t xp[10] = { 0 };")

    print("    uint64_t x2_out[4] = { 0 };")
    print("    uint64_t z2_out[4] = { 0 };")
    print("    uint64_t x3_out[4] = { 0 };")
    print("    uint64_t z3_out[4] = { 0 };")

    print("")

    print("    convert_le64_to_le25p5(x2, x2_in);")
    print("    convert_le64_to_le25p5(z2, z2_in);")
    print("    convert_le64_to_le25p5(x3, x3_in);")
    print("    convert_le64_to_le25p5(z3, z3_in);")
    print("    convert_le64_to_le25p5(xp, xp_in);")

    print("    ladder_step(x2, z2, x3, z3, xp);")

    print("    convert_le25p5_to_le64(x2_out, x2);")
    print("    convert_le25p5_to_le64(z2_out, z2);")
    print("    convert_le25p5_to_le64(x3_out, x3);")
    print("    convert_le25p5_to_le64(z3_out, z3);")
    print("    reduce_25519_le64(x2_out);")
    print("    reduce_25519_le64(z2_out);")
    print("    reduce_25519_le64(x3_out);")
    print("    reduce_25519_le64(z3_out);")

    print("")
    print("    assert(0 == memcmp(x2_out, x2_out_ref, sizeof x2_out));")
    print("    assert(0 == memcmp(z2_out, z2_out_ref, sizeof z2_out));")
    print("    assert(0 == memcmp(x3_out, x3_out_ref, sizeof x3_out));")
    print("    assert(0 == memcmp(z3_out, z3_out_ref, sizeof z3_out));")
    print("}")


def make_limb(seed):
    result = bin2int(sha256(struct.pack(">I", seed)).digest()) & ((2**255)-1)
    return result


print("#include <assert.h>")
print("#include <string.h>")
print("#include <stdint.h>")
print("#include <stdio.h>")
print("void convert_le25p5_to_le64(uint64_t out[4], const uint32_t in[10]);")
print("void convert_le64_to_le25p5(uint32_t out[10], const uint64_t in[4]);")
print("void reduce_25519_le64(uint64_t x[4]);")
print("void ladder_step(uint32_t x2[10], uint32_t z2[10], uint32_t x3[10], uint32_t z3[10], const uint32_t xp[10]);")

make_test_max()
make_test(0, 0, 0, 0, 0)
make_test(1, 1, 1, 1, 1)
make_test(6000, 10, 1000, 19999, 18888)

for x in range(50):
    x2 = make_limb(1000 + x)
    z2 = make_limb(2000 + x)
    x3 = make_limb(3000 + x)
    z3 = make_limb(4000 + x)
    xp = make_limb(5000 + x)
    make_test(x2, z2, x3, z3, xp)

make_main()



"""Make unit test for mul_25519() in x25519.c"""

from common import counter, make_main, split64, bin2int
from hashlib import sha256
import struct


def make_test(f, g):

    assert(len(f) == 10)
    assert(len(g) == 10)
    for i in range(10):
        assert(f[i] < 2**27)
        assert(g[i] < 2**27)
    fx = ["0x%08X" % x for x in f]
    gx = ["0x%08X" % x for x in g]
    max26 = hex(2**26 - 1)
    max25 = hex(2**25 - 1)

    modulus = 2**255 - 19
    base = [0, 26, 51, 77, 102, 128, 153, 179, 204, 230]
    fv = 0
    gv = 0
    for i in range(10):
        fv += f[i] * (2**(base[i]))
        gv += g[i] * (2**(base[i]))

    canonical = (fv * gv) % modulus
    results = [canonical, canonical + modulus]
    if canonical < 38:
        results.append(canonical + modulus * 2)

    # Turn results[] into arrays of 64-bit words
    results_hex = []
    for result in results:
        words = split64(result)
        words = words + ["0"] * (4 - len(words))
        results_hex.append(words)

    print("")
    print("void test_%d(void) {" % next(counter))
    print("    const uint32_t f[10] = {" + ", ".join(fx) + "};")
    print("    const uint32_t g[10] = {" + ", ".join(gx) + "};")
    print("    uint32_t out[10];")
    print("    uint64_t out64[4];")
    print("    uint64_t exp[%d][4] = {" % len(results))
    print("         { " + ",".join(results_hex[0]) + " },")
    print("         { " + ",".join(results_hex[1]) + " }")
    if len(results_hex) == 3:
        print("         ,{ " + ",".join(results_hex[2]) + " }")
    print("    };")
    print("    unsigned match;")
    print("")
    print("    mul_25519(out, f, g);")
    print("    assert(out[0] <= " + max26 + ");")
    print("    assert(out[1] <= " + max25 + ");")
    print("    assert(out[2] <= " + max26 + ");")
    print("    assert(out[3] <= " + max25 + ");")
    print("    assert(out[4] <= " + max26 + ");")
    print("    assert(out[5] <= " + max25 + ");")
    print("    assert(out[6] <= " + max26 + ");")
    print("    assert(out[7] <= " + max25 + ");")
    print("    assert(out[8] <= " + max26 + ");")
    print("    assert(out[9] <= " + max26 + ");")
    print("    convert_le25p5_to_le64(out64, out);")
    print("    match = 0;")
    print("    match |= !memcmp(exp[0], out64, 32);")
    print("    match |= !memcmp(exp[1], out64, 32);")
    if len(results_hex) == 3:
        print("    match |= !memcmp(exp[2], out64, 32);")
    print("    assert(match);")
    print("}")


def make_limb(seed):
    result = bin2int(sha256(struct.pack(">I", seed)).digest()) & ((2**27)-1)
    return result


print("#include <assert.h>")
print("#include <string.h>")
print("#include <stdint.h>")
print("#include <stdio.h>")
print("void convert_le25p5_to_le64(uint64_t out[4], const uint32_t in[10]);")
print("void mul_25519(uint32_t out[10], const uint32_t f[10], const uint32_t g[10]);")

modulus = [0x3ffffed, 0x1ffffff, 0x3ffffff, 0x1ffffff, 0x3ffffff, 0x1ffffff, 0x3ffffff, 0x1ffffff, 0x3ffffff, 0x1ffffff]
modulus_m1 = [0x3ffffec, 0x1ffffff, 0x3ffffff, 0x1ffffff, 0x3ffffff, 0x1ffffff, 0x3ffffff, 0x1ffffff, 0x3ffffff, 0x1ffffff]
modulus_m2 = [0x3ffffeb, 0x1ffffff, 0x3ffffff, 0x1ffffff, 0x3ffffff, 0x1ffffff, 0x3ffffff, 0x1ffffff, 0x3ffffff, 0x1ffffff]
modulus_m40 = [0x3ffffc5, 0x1ffffff, 0x3ffffff, 0x1ffffff, 0x3ffffff, 0x1ffffff, 0x3ffffff, 0x1ffffff, 0x3ffffff, 0x1ffffff]

make_test([0]*10, [0]*10)
make_test([1] + [0]*9, [1] + [0]*9)
make_test([30] + [0]*9, [30] + [0]*9)
make_test([0x7ffffed] + [0]*9, [0x7ffffed] + [0]*9)
make_test(modulus, modulus)
make_test(modulus_m1, modulus_m1)
make_test(modulus_m2, modulus_m2)
make_test(modulus, modulus_m2)
make_test(modulus_m40, modulus_m40)

for x in range(100):
    f = [make_limb(1000*x + y) for y in range(10)]
    g = [make_limb(2000*x + y) for y in range(10)]
    make_test(f, g)

make_main()
